/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//******************************************************************************
// MSP430FR57xx Demo - COMPD interrupt capability; Vcompare is compared against 
//                    internal 1.5V reference
//
// Description: Use CompD and internal reference to determine if input'Vcompare'
// is high of low.  For the first time, when Vcompare exceeds the 1.5V internal
// reference, CDIFG is set and device enters the CompD ISR. In the ISR, CDIES is
// toggled such that when Vcompare is less than 1.5V internal reference;
// CDIFG is set.
// LED is toggled inside the CompD ISR
//                                                   
//                 MSP430FR5739
//             ------------------                        
//         /|\|                  |                       
//          | |                  |                       
//          --|RST      P3.1/CD13|<--Vcompare
//            |                  |                                         
//            |              P3.4|----> 'high'(Vcompare>2.0V); 'low'(Vcompare<2.5V)
//            |                  |  |
//            |                  |  | LED 'ON'(Vcompare>2.0V); 'OFF'(Vcompare<2.5V)
//            |                  | 
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/compd.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

void main (void)
{
    //Stop WDT
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

    //Set P3.4 as an output pin.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 4 as output
     */
  	FRGPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN4
  	);
    //Set P3.4 as Output Low.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 4 to output Low.
     */
    FRGPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN4
  	);

    //Initialize the Comparator B module
    /*
     * Base Address of Comparator B,
     * Pin CD13 to Positive(+) Terminal
     * Reference Voltage to Negative(-) Terminal
     * Normal Power Mode
     * Output Filter On with minimal delay
     * Non-Inverted Output Polarity
     */
    COMPD_init(__MSP430_BASEADDRESS_COMPD__,
        COMPD_INPUT13,
        COMPD_VREF,
        COMPD_FILTEROUTPUT_OFF,
        COMPD_NORMALOUTPUTPOLARITY
        );

    //Set the reference voltage that is being supplied to the (-) terminal
    /*
     * Base Address of Comparator D,
     * Reference Voltage of 1.5 V,
     * Upper Limit of 1.5*(32/32) = 1.5V,
     * Lower Limit of 1.5*(32/32) = 1.5V
     */
    COMPD_setReferenceVoltage(__MSP430_BASEADDRESS_COMPD__,
        COMPD_VREFBASE1_5V,
        32,
        32
        );

    //Enable CompB Interrupt on default rising edge for CDIFG
    COMPD_interruptSetEdgeDirection(__MSP430_BASEADDRESS_COMPD__, COMPD_RISINGEDGE);

    // Clear any erroneous interrupts
    COMPD_clearInterrupt(__MSP430_BASEADDRESS_COMPD__, (COMPD_INTERRUPT_FLAG + COMPD_INTERRUPT_FLAG_INVERTED_POLARITY));

    //Enable Interrupts
    /*
     * Base Address of Comparator B,
     * Enable CompB Interrupt on default rising edge for CDIFG
     */
    COMPD_enableInterrupt(__MSP430_BASEADDRESS_COMPD__,
        COMPD_INTERRUPT_ENABLE
        );

    //Allow power to Comparator module
    COMPD_enable(__MSP430_BASEADDRESS_COMPD__);

    //Enter LPM4 with interrupts enabled
    __bis_SR_register(LPM4_bits + GIE);

    //For debug
    __no_operation();
}

//******************************************************************************
//
//This is the COMP_B_VECTOR interrupt vector service routine.
//
//******************************************************************************
#pragma vector=COMP_D_VECTOR
__interrupt void COMPD_D_ISR (void)
{
    //Toggle the edge at which an interrupt is generated
    COMPD_interruptToggleEdgeDirection(__MSP430_BASEADDRESS_COMPD__);

    //Clear Interrupt flag
    COMPD_clearInterrupt(__MSP430_BASEADDRESS_COMPD__, COMPD_INTERRUPT_FLAG);

	//Toggle P3.4 output pin.
	/*
	* Base Address for Port 3
	* Select Port 3
	* Set Pin 4 as output
	*/
	FRGPIO_toggleOutputOnPin(__MSP430_BASEADDRESS_PORT3_R__,
		FRGPIO_PORT_P3,
		FRGPIO_PIN4
	);
}
