/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430F51x2 Demo - Timer0_D3, Combining two CCRx register to control one 
//                     PWM channel; Normal Timer mode
//
//  Description: This code example shows how to combine two CCRx registers to 
//  control both the rising and falling edges of one PWM channel (TD0.2, TD1.2). 
//  In up-mode, TDxCCR1 controls the rising edge and TDxCCR2 controls the 
//  falling edge. In this example, since CCR1 registers of TD0/1 have the same 
//  count, the rising edges at TD0.2 and TD1.2 happen almost at the same 
//  time instance. With TD0CCR2=40 and TD1CCR2=96, the dutycycles of the TD0.2 
//  and TD1.2 are 30% and 43.75%. 
//  
//  ACLK = REF0; SMCLK = MCLK = default DCO ~1.045MHz.
//
//                 MSP430F51x2
//             -----------------
//         /|\|              XIN|-
//          | |                 | 
//          --|RST          XOUT|-
//            |                 |
//            |       P2.0/TD0.2|--> CCR2 - 30% duty cycle; 
//            |                 |    ON period=(TD0CCR2-TD0CCR1)/32767 ~ 1.22ms
//            |       P2.3/TD1.2|--> CCR2 - 43.75% duty cycle; 
//            |                 |    ON period=(TD1CCR2-TD1CCR1)/32767 ~ 1.7ms 
//
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/timerd.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"

void main(void)
{
  WDT_hold(__MSP430_BASEADDRESS_WDT_A__);
  
  // Set port pins
  GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT2_R__,
        		  GPIO_PORT_P2,
        		  GPIO_PIN0 + GPIO_PIN3
        		  );


  // Configure TimerD0 to combine CCR0/1 block to control TD0CCR2 PWM output
  TimerD_combineTDCCRToGeneratePWM(__MSP430_BASEADDRESS_T0D3__,
		  TIMERD_CLOCKSOURCE_SMCLK,
		  TIMERD_CLOCKSOURCE_DIVIDER_1,
		  TIMERD_CLOCKINGMODE_EXTERNAL_CLOCK,
                  128,
		  TIMERD_COMBINE_CCR1_CCR2,
		  TIMERD_OUTPUTMODE_RESET_SET,
		  40,
		  80
		  );
  
  // Configure TimerD1 to combine CCR0/1 block to control TD1CCR2 PWM output
  TimerD_combineTDCCRToGeneratePWM(__MSP430_BASEADDRESS_T1D3__,
		  TIMERD_CLOCKSOURCE_SMCLK,
		  TIMERD_CLOCKSOURCE_DIVIDER_1,
		  TIMERD_CLOCKINGMODE_EXTERNAL_CLOCK,
  		  128,
  		  TIMERD_COMBINE_CCR1_CCR2,
  		  TIMERD_OUTPUTMODE_RESET_SET,
  		  40,
  		  96
  		  );


  TimerD_configureUpMode(__MSP430_BASEADDRESS_T0D3__,
    	      TIMERD_CLOCKSOURCE_ACLK,
              TIMERD_CLOCKSOURCE_DIVIDER_1,
              TIMERD_CLOCKINGMODE_EXTERNAL_CLOCK,
              128-1,
              TIMERD_TDIE_INTERRUPT_DISABLE,
              TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
              TIMERD_DO_CLEAR
              );

  TimerD_configureUpMode(__MSP430_BASEADDRESS_T1D3__,
    		 TIMERD_CLOCKSOURCE_ACLK,
              TIMERD_CLOCKSOURCE_DIVIDER_1,
              TIMERD_CLOCKINGMODE_EXTERNAL_CLOCK,
              128-1,
              TIMERD_TDIE_INTERRUPT_DISABLE,
              TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
              TIMERD_DO_CLEAR
              );
  TimerD_startCounter(__MSP430_BASEADDRESS_T0D3__,
		  TIMERD_UP_MODE
		  );
  TimerD_startCounter(__MSP430_BASEADDRESS_T1D3__,
		  TIMERD_UP_MODE
  		  );

  __bis_SR_register(LPM0_bits);             // Enter LPM0
  __no_operation();                         // For debugger    
  
}
